//
//  FetchGiphyAPIImp.swift
//  LiveLikeGiphyChallenge
//
//  Created by Irina Smokvarska on 1/30/22.
//

import Foundation
import UIKit

public final class FetchGiphyAPI {
  
  var urlSession = URLSession.shared
  private var path = ""
  
  func fetchTrendingGifs(chunk: Int) async throws -> [GiphyData] {
    path = "\(Constants.trendingUrlPath)?api_key=\(Constants.apiKey)&limit==\(chunk)&offset=0"
    do {
      let response = try await httpGet(responseType: GiphyResponse.self, timeout: 10, chunk: chunk)
      return response.data
    } catch {
      throw GiphyError.errorNetwork
    }
  }
  
  func fetchGifsBy(string: String, chunk: Int) async throws -> [GiphyData] {
    path = "\(Constants.searchUrlPath)?api_key=\(Constants.apiKey)&q=\(string)&limit=\(chunk)&offset=0&lang=en"
    do {
      let response = try await httpGet(responseType: GiphyResponse.self, timeout: 10, chunk: chunk)
      return response.data
    } catch {
      throw GiphyError.errorNetwork
    }
  }
  
  func httpGet<Response: Decodable>(
    responseType: Response.Type,
    timeout: TimeInterval,
    chunk: Int) async throws -> Response {
      guard let url = URL(string: path) else { throw GiphyError.errorConvertingToUrl }
      var request = URLRequest(url: url, timeoutInterval: timeout)
      request.httpMethod = "GET"

      return try await withCheckedThrowingContinuation { continuation in
        let task = URLSession.shared.dataTask(with: request) { (data1, response, error) in
          
          if let error = error {
            continuation.resume(with: .failure(error))
            return
          }
          
          if let data = data1 {
            do {
              let response = try JSONDecoder().decode(Response.self, from: data)
              continuation.resume(with: .success(response))
            } catch {
              continuation.resume(with: .failure(error))
            }
          }
        }
        task.resume()
      }
    }
}
