//
//  ContentView.swift
//  LiveLikeGiphyChallenge
//
//  Created by Irina Smokvarska on 1/30/22.
//

import SwiftUI

struct ContentView: View {
  
  @ObservedObject var viewModel: MainViewModel
  
  var body: some View {
    
    NavigationView {
      List {
        ForEach(viewModel.searchResults, id: \.self) { data in
          VStack {
            GiphyItemRow(url: URL(string: data.images.original.url)!)
              .frame(height: 100, alignment: .leading)
              .frame(minWidth: 0, maxWidth: .infinity)
              .onAppear {
                viewModel.increaseLoadedGifCount()
                if viewModel.loadedGifCount == viewModel.chunkCount {
                  viewModel.increaseChunkCount()
                  viewModel.fetchTrendingData(chunk: viewModel.chunkCount)
                }
              }
          }
        }
      }
    }
    .alert(isPresented: $viewModel.hasErrorWhileFetching) {
      Alert(
        title: Text("Error"),
        message: Text("Something went wrong while fetching gifs"),
        dismissButton: .cancel(Text("Ok"))
      )
    }
    .searchable(text: $viewModel.searchText)
  }
}

