//
//  MainViewModel.swift
//  LiveLikeGiphyChallenge
//
//  Created by Irina Smokvarska on 1/30/22.
//

import Foundation
import Combine
import SwiftUI

@MainActor
final class MainViewModel: ObservableObject {
  
  @Published var loadedGifCount = 0
  @Published var chunkCount: Int = 20
  @Published var hasErrorWhileFetching = false
  @Published var searchResults: [GiphyData] = [GiphyData]()
  @Published var searchText = ""
  
  private var subscriptions = Set<AnyCancellable>()
  private let api: FetchGiphyAPI = FetchGiphyAPI()
  
  init() {
    $searchText.sink { [weak self] stringValue in
      if stringValue.isEmpty {
         self?.fetchTrendingData(chunk: self?.chunkCount ?? 0)
      } else {
        self?.fetchSearchDataBy(string: self!.searchText , chunk: self?.chunkCount ?? 0)
      }
    }
    .store(in: &subscriptions)
  }
  
  func fetchTrendingData(chunk: Int) {
    Task {
      do {
        guard Storage.fileExists("data.json", in: .documents) else {
          self.searchResults = try await api.fetchTrendingGifs(chunk: self.chunkCount)
          Storage.store(self.searchResults, to: .documents, as: "data.json")
          return
        }
        
        self.searchResults = Storage.retrieve("data.json", from: .documents, as: [GiphyData].self)
        let difference = self.chunkCount - self.searchResults.count
        if self.searchResults.count <= self.chunkCount && difference == 20  {
          self.searchResults = try await api.fetchTrendingGifs(chunk: self.chunkCount)
          Storage.store(self.searchResults, to: .documents, as: "data.json")
        }
      } catch {
        print("Error fetching trening data")
        hasErrorWhileFetching = true
      }
    }
  }

  func increaseChunkCount() {
    chunkCount+=20
  }
  
  func increaseLoadedGifCount() {
    loadedGifCount+=1
  }
}

//MARK: Private
private extension MainViewModel {
  func fetchSearchDataBy(string: String, chunk: Int) {
    Task {
      do {
        self.searchResults = try await api.fetchGifsBy(string: string, chunk: chunkCount)
      } catch {
        print("Error fetching search data")
        hasErrorWhileFetching = true
      }
    }
  }
}
  
  



